@@ -18,7 +18,14 @@
 #
 #    (C) 2005 Nicolas Évrard <nicoe@altern.org>
 
-__revision__ = "$Id: gnomolicious.py,v 1.6 2005/02/02 11:52:14 nicoe Exp $"
+__revision__ = "$Id: gnomolicious.py,v 1.8 2005/02/03 14:44:25 nicoe Exp $"
+
+import os.path
+import re
+import logging
+import urllib
+import ConfigParser
+from time import time as now
 
 import pygtk
 pygtk.require('2.0')
@@ -29,12 +36,6 @@ import gnome.ui
 import gconf
 import gobject
 
-import os.path
-import re
-import logging
-import urllib
-from time import time as now
-
 import delicious
 import conf
 
@@ -49,58 +50,87 @@ def findMatchingTag(completion, key_string, cursor, column):
 class GnomoliciousConfiguration(object):
     
     GCONF_PATH = "/apps/gnomolicious"
+    PASSWORD_PATH = os.path.expanduser('~/.gnome2_private/gnomolicious')
     
     def __init__(self):
-        self.__gconf = gconf.client_get_default()
-        self.__gconf.add_dir(self.GCONF_PATH, gconf.CLIENT_PRELOAD_RECURSIVE)
+        self.privconf = ConfigParser.ConfigParser()
+        self.privconf.read([self.PASSWORD_PATH])
+        
+        self.gconf = gconf.client_get_default()
+        self.gconf.add_dir(self.GCONF_PATH, gconf.CLIENT_PRELOAD_RECURSIVE)
+        self.conf = {}
+        self.conf['update_interval'] = \
+                self.gconf.get_int(self.GCONF_PATH + '/update_interval')
+        self.conf['common_tag_color'] = \
+                self.gconf.get_string(self.GCONF_PATH + '/common_tag_color')
+        self.conf['pop_tag_num'] = \
+                self.gconf.get_int(self.GCONF_PATH + '/pop_tag_num')
+        self.gconf.notify_add(self.GCONF_PATH, self.changed_conf)
+        
+    def changed_conf(self, cid, entry, cdata = None):
+        key, value = entry.key, entry.value
+        keyname = key.split('/')[-1]
+        self.conf[keyname] = value
         
     def getUsername(self):
-        return self.__gconf.get_string(self.GCONF_PATH + '/username')
+        try:
+            return self.privconf.get('delicious', 'username')
+        except (ConfigParser.NoSectionError, ConfigParser.NoOptionError):
+            return ''
 
     def setUsername(self, user):
-        self.__gconf.set_string(self.GCONF_PATH + '/username', user)
+        if not self.privconf.has_section('delicious'):
+            self.privconf.add_section('delicious')
+        self.privconf.set('delicious', 'username', user)
+        self.privconf.write(open(self.PASSWORD_PATH, 'w'))
 
     username = property(getUsername, setUsername)
 
     def getPassword(self):
-        return self.__gconf.get_string(self.GCONF_PATH + '/password')
+        try:
+            return self.privconf.get('delicious', 'password')
+        except (ConfigParser.NoSectionError, ConfigParser.NoOptionError):
+            return ''
 
     def setPassword(self, password):
-        self.__gconf.set_string(self.GCONF_PATH + '/password', password)
+        if not self.privconf.has_section('delicious'):
+            self.privconf.add_section('delicious')
+        self.privconf.set('delicious', 'password', password)
+        self.privconf.write(open(self.PASSWORD_PATH, 'w'))
 
     password = property(getPassword, setPassword)
 
     def getUpdateInterval(self):
-        return self.__gconf.get_int(self.GCONF_PATH + '/update_interval')
+        return self.conf['update_interval']
 
     def setUpdateInterval(self, interval):
-        self.__gconf.set_int(self.GCONF_PATH + '/update_interval', interval)
+        self.gconf.set_int(self.GCONF_PATH + '/update_interval', interval)
+        self.gconf.suggest_sync()
 
     update_interval = property(getUpdateInterval, setUpdateInterval)
 
     def getCommonTagColor(self):
-        return self.__gconf.get_string(self.GCONF_PATH + '/common_tag_color')
+        return self.conf['common_tag_color']
 
     def setCommonTagColor(self, color):
-        self.__gconf.set_string(self.GCONF_PATH + '/common_tag_color',
+        self.gconf.set_string(self.GCONF_PATH + '/common_tag_color',
                 color)
+        self.gconf.suggest_sync()
 
     common_tag_color = property(getCommonTagColor, setCommonTagColor)
 
     def getPopTagNum(self):
-        return self.__gconf.get_int(self.GCONF_PATH + '/pop_tag_num')
+        return self.conf['pop_tag_num']
 
     def setPopTagNum(self, num):
-        self.__gconf.set_int(self.GCONF_PATH + '/pop_tag_num', num)
+        self.gconf.set_int(self.GCONF_PATH + '/pop_tag_num', num)
+        self.gconf.suggest_sync()
 
     pop_tag_num = property(getPopTagNum, setPopTagNum)
 
-    def sync(self):
-        self.__gconf.suggest_sync()
-
 class GnomoliciousApplet(gnome.applet.Applet):
     
-    title_re = re.compile('<title>(?P<title>.*)</title>',
+    title_re = re.compile('<title>(?P<title>.*?)</title>',
             re.IGNORECASE|re.DOTALL)
     
     def __init__(self, applet, iid):
@@ -113,7 +143,7 @@ class GnomoliciousApplet(gnome.applet.Applet):
         self.conf = GnomoliciousConfiguration()
         
         # Popup-menu definition and popup-menu callbacks
-        self.propxml="""\
+        self.propxml = """\
 <popup name="button3">
   <menuitem name="post" verb="Post" label="%s"
     pixtype="stock" pixname="gtk-jump-to" />
@@ -169,7 +199,9 @@ class GnomoliciousApplet(gnome.applet.Applet):
         self.postwin.completion = gtk.EntryCompletion()
         self.postwin.entries['tags'].set_completion(self.postwin.completion)
         self.tags = gtk.ListStore(str, str)
-        self.update_tags()
+        # TODO: prefetch tag ? can slow gnome startup when del.icio.us is
+        # slow
+        # self.update_tags()
         self.postwin.completion.set_model(self.tags)
         numtags = gtk.CellRendererText()
         numtags.set_property('foreground', 'grey')
@@ -198,7 +230,8 @@ class GnomoliciousApplet(gnome.applet.Applet):
 
     def update_tags(self):
         if (self.conf.username and self.conf.password and
-            (now() - self.last_update) > self.conf.update_interval*60) :
+            ((now() - self.last_update) > self.conf.update_interval*60)
+             or not list(self.tags)):
             logger.info('Updating your tags')
             self.last_update = now()
             self.tags.clear()
@@ -223,7 +256,7 @@ class GnomoliciousApplet(gnome.applet.Applet):
             self.postwin.entries['popular'].remove(btn)
         self.populartags_buttons = {}
         self.postwin.entries['popular'].attach(self.populartags_nobuttons,
-                0, 15, 0, 5)
+                0, 3, 0, (self.conf.pop_tag_num - 1) / 3 + 1, 2, 2)
 
     def clear_prefwin(self):
         pass
@@ -235,7 +268,7 @@ class GnomoliciousApplet(gnome.applet.Applet):
             for tag in post['tags'].split():
                 tags[tag] = tags.get(tag, 0) + 1
         lst = tags.items()
-        lst.sort(lambda x,y: cmp(x[1], y[1]))
+        lst.sort(lambda x, y: cmp(x[1], y[1]))
         lst.reverse()
         return lst
 
@@ -323,7 +356,7 @@ class GnomoliciousApplet(gnome.applet.Applet):
         self.update_tags()
         logger.info('Retrieving popular tags')
         poptags = self.get_poptags(url)
-        self.add_poptags(poptags[:15])
+        self.add_poptags(poptags[:self.conf.pop_tag_num])
         self.postwin.show_all()
 
     def on_button_press(self, widget, evt):
@@ -349,13 +382,14 @@ class GnomoliciousApplet(gnome.applet.Applet):
     def on_setpref(self, evt, data=None):
         self.conf.username = self.prefwin.entries['username'].get_text()
         self.conf.password = self.prefwin.entries['password'].get_text()
-        self.conf.update_interval = self.prefwin.entries['updatetime'].get_value_as_int()
+        self.conf.update_interval = \
+                self.prefwin.entries['updatetime'].get_value_as_int()
         color = self.prefwin.entries['commontagcolor'].get_color()
         self.conf.common_tag_color = '#%04x%04x%04x' % (color.red,
                 color.green, color.blue)
-        self.conf.pop_tag_num = self.prefwin.entries['maxpoptag'].get_value_as_int()
+        self.conf.pop_tag_num = \
+                self.prefwin.entries['maxpoptag'].get_value_as_int()
 
-        self.conf.sync()
         self.prefwin.hide()
         return gtk.TRUE
 
@@ -380,14 +414,11 @@ class GnomoliciousApplet(gnome.applet.Applet):
         self.postwin.entries['tags'].set_text(' '.join(etokens))
         return gtk.TRUE
 
-    def on_postwin_tags(self, widget, event):
-        if event.string == ' ':
-            tags = self.postwin.entries['tags'].get_text().split()
-            for tag in tags:
-                if tag in self.populartags_buttons:
-                    btn = self.populartags_buttons[tag]
-                    btn.set_active(True)
-        return gtk.FALSE
+    def on_postwin_tags(self, widget):
+        tags = self.postwin.entries['tags'].get_text().split()
+        for tag, btn in self.populartags_buttons.items():
+            btn.set_active(tag in tags)
+        return gtk.TRUE
 
     def on_register_clicked(self, evt, data=None):
         gnome.url_show('http://del.icio.us/register')
